@@ -1,20 +1,18 @@
 /* bigram -- list bigrams for locate
-   Copyright (C) 1994, 2000, 2003, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1994, 2000, 2003, 2004, 2005, 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-   USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* Usage: bigram < text > bigrams
@@ -45,7 +43,6 @@
 #endif
 #include <sys/types.h>
 
-#include <getline.h>
 #include <xalloc.h>
 #include "closeout.h"
 
@@ -75,14 +72,13 @@ main (int argc, char **argv)
   program_name = argv[0];
   (void) argc;
   atexit (close_stdout);
-  
+
   pathsize = oldpathsize = 1026; /* Increased as necessary by getline.  */
   path = xmalloc (pathsize);
   oldpath = xmalloc (oldpathsize);
 
-  /* Set to anything not starting with a slash, to force the first
-     prefix count to 0.  */
-  strcpy (oldpath, " ");
+  /* Set to empty string, to force the first prefix count to 0.  */
+  oldpath[0] = '\0';
 
   while ((line_len = getline (&path, &pathsize, stdin)) > 0)
     {
